#!/usr/local/bin/perl
#
# USAGE:   affixes dict ... > dict.freq
#
# PURPOSE: count most frequently used affixes in a given dictionary
#	   or in a munched (munchlist) dictionary. Program shows only
#	   affixes wich occur more frequent than the value of $THRESHOLD
#          (0.4%). For those affixes 5 samples are shown.
#	   Program needs a lot of memory depending on the size of the 
#	   used dictionary.
# 
# AUTHOR:  Piet Tutelaers <rcpt@urc.tue.nl>
# VERSION: 1.0 (Aug. 1997)

$HYPHEN = "\267";
$MIN = 3;
$MAX = 6;
$THRESHOLD = 0.4;
$SAMPLES = 5;

%dict = (); %affix = ();

print STDERR "Reading words ...";
while (<>) {
   chop;
   s/$HYPHEN//g; # remove hyphens
   $dict{$_} = 1;
   $words++;
}
print STDERR " done\n";

print STDERR "Counting affixes ...";
foreach (keys(%dict)) {
   s,/.*,,; 	# strip off the flags
   $l = length($_);
   for ($i = 1; $i < $l-$MIN && $i <= $MAX; $i++) {
      $tail = substr($_, $i);
      $pref = substr($_, 0, $i);
      if ($dict{$tail}) {
	 $affix{".$pref"}++;
	 $example{".$pref"} .= "$pref+$tail "
	    unless  split(/ /, $example{".$pref"}) == $SAMPLES;
      }
      $suf  = substr($_, -$i);
      $head = substr($_, 0, $l-$i);
      if ($dict{$head}) {
	 $affix{"$suf."}++;
	 $example{"$suf."} .= "$head+$suf "
	    unless  split(/ /, $example{"$suf."}) == $SAMPLES;
      }
   }
}
print STDERR " done\n";

print STDERR "Writing most frequently used affixes ...";
foreach $key (sort { $affix{$b} <=> $affix{$a} } (keys %affix)) {
   $perc = $affix{$key}/$words*100.0;
   (@examples) = split(/ /, $example{$key}, $SAMPLES+1);
   if ($key =~ /^[.]/) {
      $key =~ s/^.//;
      print sprintf("%-8s\t# %8.2f%%\t%s\n", uc $key, $perc, $examples[0]);
   }
   else {
      $key =~ s/.$//;
      print sprintf("%8s\t# %8.2f%%\t%s\n", uc $key, $perc, $examples[0]);
   }
   shift @examples;
   foreach (@examples) {
      next if /^$/;
      print sprintf("%8s\t# %8s \t%s\n", "", "", $_);
   }
   last if $perc < $THRESHOLD;
}
print STDERR " done\n";
